#### R script ########################################
# Last modfied: 2019-10-02
# File name: irf_two_methods_VAR.R
# Keywords for file name 
#   irf: impulse response function, 
#   two methods: two methods, 
#        (1) orthogonal impulse response function, (2) general impulse response function 
#   VAR: use a VAR model
#
# Written by: Hiroshi Murao
#
# Purpose: 
#   Performe impulse response analyses 
#   so that we get Fig.1 and Fig.2 shown in the following journal paper.
#
#   Pesaran, H. H. and Y. Shin (1998), 
#   "Generalized Impulse Response Analysis in Linear Multivariate Models,"
#   Economics Letters, Vol. 58, 17-29.
#
# Data source:
#   The original data set is the "KPSW" data, 
#   which is used in the following journal paper. 
#
#   King, Robert G., Charles I. Plosser, James H. Stock, 
#   and Mark W. Watson (1991),
#   "Stochastic Trends and Economic Fluctuations," 
#   The American Economic Review, 81:4, 819-840.
# 	
#   Our "KPSW" data is obtained from TSP program "gir.tsp" 
#   at Clint's home page.
#   (http://www.stanford.edu/~clint/tspex/gir2.tsp)
#   The program "gir.tsp" can not be found on the Internet (2019-09-29).
#
# Data:	
#   Data type: U.S. quarterly data
#   Time periods: 1948:1 - 1988:4 
#   Variable names (the same names are used with "gir.tsp")		
#     ey = per capita "private" gross national product, 
#          defined as total gross national product. 
#          less real total government purchases of goods and services.		
#     ec = per capita real consumption expenditures.		
#     ein = per capita real investment expenditures.
#
##########################################################
#### first things to do 
setwd("F:/book_VAR/data_works/Pesaran_EL1998/R_prg")
library(vars) # VAR
library(urca) # unit root, cointegration
search()

#### get data
in_data <- read.csv("./data/data_KPSW.csv")
print(in_data)

logY <- log(in_data$ey)
logC <- log(in_data$ec)
logI <- log(in_data$ein)

#### make a data set for VAR()
y_data_VAR <- data.frame(y1 = logI, y2 = logC, y3 = logY)

#### estimate a VAR model, using VAR()
out_VAR <- VAR(y = y_data_VAR, p = 4, type = "both")

#### impulse response analysis, using irf() in "vars"
out_irf_VAR <- irf(x = out_VAR, n.ahead = 50, boot = TRUE, ci = 0.95)

#### plot graphs with the output data of irf()
plot.new()
plot(out_irf_VAR)

#### check the content of out_VAR
str(out_VAR)
# print(out_VAR)
# print(Acoef(out_VAR))
print(Bcoef(out_VAR))
# print(coef(out_VAR))
print(summary(out_VAR))

#### get estimates from out_VAR
# get coefficient estimates
B_whole <- Bcoef(out_VAR)
print(B_whole)
B_1 <- B_whole[,1:3]
B_2 <- B_whole[,4:6]
B_3 <- B_whole[,7:9]
B_4 <- B_whole[,10:12]
B_z <- B_whole[,13:14]
print(B_1)
print(B_z)

# get estimates for sigma_u
str(summary(out_VAR))
out_summary <- summary(out_VAR)
sigma_cov <- out_summary$covres
print(sigma_cov)  # use this as sigma_u

resid <- residuals(out_VAR)
sigma_resid <- cov(resid, y = resid)
print(sigma_resid) # not use as sigma_u

# choose one for sigma_u
sigma_u <- sigma_cov 

# selection vector 
select <- c(0, 0, 1)  # 3x1 vector
print(select)

# sacle factor, change if necessary
scale_factor <- 1

#### shock for the orthogonal impulse response function 
# Cholesky facorization, get the lower triangular matrix
chol_P <- t(chol(sigma_u)) 
# make sure that chol_P is a lower triangular matrix.
print(chol_P)

shock_orth <- chol_P%*%select*scale_factor  
# (3x3)%*%(3x1) -> 3x1 vector
print(shock_orth)  

#### shock for the general impulse response function
sigma_pick <- sigma_u[3,3]
sqrt_sigma <- sqrt(sigma_pick)

shock_gen <- (sigma_u/sqrt_sigma)%*%select*scale_factor
print(shock_gen)

#### set important numbers
num_y <- 3
num_z <- 2
num_lag <- 4
num_x <- num_lag*num_y + num_z
num_h <- 50  # size of horizon
num_col <- num_h + num_lag

#### loop for orthogonalised impulse response
y <- matrix(0, nrow = num_y, ncol = num_col)
z <- matrix(0, nrow = num_z, ncol = num_col)

# initial scock at h = 1
h <- num_lag + 1
y[, h] <- shock_orth

for (h in (num_lag + 2):num_col){
y[, h] <- B_1%*%y[, h-1] + B_2%*%y[, h-2] + B_3%*%y[, h-3] + B_4%*%y[, h-4] + B_z%*%z[, h]
}
#### end of h-loop

print(y) 

#### prepare for Fig. 2
out_y <- y[, (num_lag-1):num_col]
irf_orth <- t(out_y)  # transpose for graph
print(nrow(irf_orth))

y_min <- min(irf_orth)
y_max <- max(irf_orth)
print(c(y_min, y_max))

x_data <- matrix(-1:num_h, nrow = (num_h + 2), ncol = 1)
print(nrow(x_data))

#### plot Fig.2 of in Pesaran and Shin (1998)
plot.new()

matplot(x_data, irf_orth, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted"), 
xlim = c(-1, 50), ylim = c(-0.015, 0.010), 
xlab = "Time Horizon", ylab = "Impulse Response", 
main = "Orthogonal Impulse Responses: impulse = output")

# add a zero-line to the graph
abline(h = 0, col = "gray")

#### save the graph as a file
dev.copy(jpeg, file = "./outputs/Persaran_Fig2.jpg")
dev.off()

#### loop for generalised impulse response
y <- matrix(0, nrow = num_y, ncol = num_col)
z <- matrix(0, nrow = num_z, ncol = num_col)

# initial scock at h = 1
h <- num_lag + 1
y[, h] <- shock_gen

for (h in (num_lag + 2):num_col){
y[, h] <- B_1%*%y[, h-1] + B_2%*%y[, h-2] + B_3%*%y[, h-3] + B_4%*%y[, h-4] + B_z%*%z[, h]
}
#### end of h-loop 

print(y)

#### preparing for Fig. 1
out_y <- y[, (num_lag - 1):num_col]
irf_gen <- t(out_y)  # transpose for graph
print(nrow(irf_gen))

y_min <- min(irf_gen)
y_max <- max(irf_gen)
print(c(y_min, y_max))

x_data <- matrix(-1:num_h, nrow = (num_h + 2), ncol = 1)
print(nrow(x_data))

#### plot Fig.1 of Pesaran and Shin (1998)
plot.new()

matplot(x_data, irf_gen, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted"), 
xlim = c(-1, 50), ylim = c(-0.010, 0.025), 
xlab = "Time Horizon", ylab = "Impulse Response", 
main="General Impulse Response: impulse = output")

# add a zero-line to the graph
abline(h = 0, col = "gray")

#### save the graph as a file
dev.copy(jpeg, file = "./outputs/Persaran_Fig1.jpg")
dev.off()

###### part 2 of this program ##########################  
#### plot Fig. 2, using irf()
out_irf_orth <- irf(out_VAR, impulse = "y3", 
response = c("y1", "y2", "y3"), n.ahead = num_h, boot = FALSE)

#### see what we get in out_irf_orth
plot(out_irf_orth)
print(summary(out_irf_orth))
str(summary(out_irf_orth))
print(out_irf_orth$irf)

#### important line. Need to get IRF values in a matrix. 
out_irf_orth_mat <- out_irf_orth$irf$y3[,1:3]     
print(out_irf_orth_mat)
str(summary(out_irf_orth_mat))

#### preparing for Fig. 2
ini_0 <- matrix(0, nrow = 1, ncol = num_y)
# add initial values so that the graph starts from (x,y) = (0,0) 
irf_orth_byR <- rbind(ini_0, out_irf_orth_mat)   
print(irf_orth_byR)
print(nrow(irf_orth_byR))

y_min <- min(irf_orth_byR)
y_max <- max(irf_orth_byR)
print(c(y_min, y_max))

x_data <- matrix(-1:num_h, nrow = (num_h + 2), ncol = 1)
print(nrow(x_data))

#### plot Fig.2
plot.new()

matplot(x_data, irf_orth_byR, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted"), 
xlim = c(-1, 50), ylim = c(-0.015, 0.010), 
xlab = "Time Horizon", ylab = "Impulse Response", 
main = "Orthogonal Impulse Response: use of irf()")

# add a zero-line to the graph
abline(h = 0, col = "gray")

# save the graph as a file
# dev.copy(jpeg, file = "./outputs/Persaran_Fig2_byR.jpg")
# dev.off()

#### end #################################################
