#### R script ########################################
# Last modfied: 2019-05-26
# File name: irf_SVEC_vars.R
# Keywords: impulse response analysis,
#   using SVEC() function in Package "vars", 
#  
# Written by: Hiroshi Murao
#
# Purpose: perform the impulse response analysis based on a SVEC model,
#   using SVEC() function in Package "vars".
#
# Related journal paper: Breitung J, Bruggemann R, Lutkepohl H (2004) 
# "Structural Vector Autoregressive Modeling and Impulse Responses". 
# In Lutkepohl H, Kratzig M (eds), "Applied Time Series Econometrics", 
# chapter 4, pp.159-196, Cambridge University Press, 
# ISBN 0-521-83919-X (hardback). - ISBN 0-521-54787-3 (paperback)  
#
# Name of data set: "Canada" in Package "vars".
#
# Sample periods:  1980:1 2004:4
#
# List of variables
#
# prod : labor productivity = log of GDP - log of employment
# e : log of employment
# U : unemplyment rate
# rw : real wage = log of real wage index 
#
#### first things to do 
# setwd("C:/school/stat_homework/time_VAR/Breitung/R_prg")
setwd("F:/book_VAR/stat_works/Pfaff_Canada/R_prg")
library(vars)

#### get data 
data(Canada)
print("Check the data@set of Canada")
print(Canada)
# summary(Canada)

#### Johansen procedure for VAR, using ca.jo()
out_cajo <- ca.jo(Canada[, c("prod", "e", "U", "rw")], 
  type = "trace", ecdet = "trend", K = 3, spec = "transitory")

### make the long run impact matrix
LR <- matrix(NA, nrow = 4, ncol = 4)
# a cointegration relation is interpreted as a stationary wage-setting relation.
# the column corresponding to "rw" has zero entries.
LR[1:4, 4] <- 0
# no long-run impact from ("e", "U", "rw") to "prod".
LR[1, 2:4] <- 0

#### make the contemporaneous (short run) impact matrix
SR <- matrix(NA, nrow = 4, ncol = 4)
# no contemporaneous impact from "e" to "rw".
SR[4, 2] <- 0

#### estimate a SVEC model, using SVEC()
out_SVEC <- SVEC(out_cajo, r = 1, LR = LR, SR = SR, 
  lrtest = FALSE, boot = TRUE, runs = 100)
summary(out_SVEC)

#### impulse response function, using irf()
# response = "U" = y3
irf_y3_ss <- irf(out_SVEC, response = "U", n.ahead = 48, boot = TRUE)

#### plot graphs with the output data of irf()
plot.new()
plot(irf_y3_ss)

#### choose some data for Panel 1 of Figure 5, Pfaff (2008).
# choose data such that impulse = "pord" = y1
names(irf_y3_ss)
names(irf_y3_ss$irf)
print(irf_y3_ss$irf$prod)

irf_y3_s1_mean <- irf_y3_ss$irf$prod
irf_y3_s1_lo <- irf_y3_ss$Lower$prod
irf_y3_s1_up <- irf_y3_ss$Upper$prod

#### preparing for plotting
# get y_data
y_data <- cbind(irf_y3_s1_mean, irf_y3_s1_lo, irf_y3_s1_up)
colnames(y_data) <- c("mean", "lower", "upper")
# note that the values at the first row are not zero.
print(y_data)
# make initial (zero) values at the first row.
y_data <- rbind(c(0, 0, 0), y_data)
print(y_data)

# get x_data
num_row <- nrow(y_data)
x_data <- matrix(-1:(num_row - 2), nrow = num_row, ncol = 1)
print(x_data)

# min, max information
x_max <- max(x_data)
x_min <- min(x_data)
print(c(x_min, x_max))

y_min <- min(y_data[, 2])
y_max <- max(y_data[, 3])
print(c(y_min, y_max))

#### plot, using matplot().
plot.new()

matplot(x_data, y_data, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dotted", "dotted"), 
xlim = c(-1, (num_H + 15)), 
ylim = c(-0.60, 0.40), 
xlab = "time horizon (quarter)", 
ylab = "impulse response (persent)", 
main = "impulse response: response = U = y3, shock = prod = y1. Use of SVEC()")

# additional infromation to the graph
abline(h = 0, col = "gray")
text(x_max, y_data[(x_data == x_max), 1], labels = "irf_y3_s1", pos = 4)
text(x_max, y_data[(x_data == x_max), 2], labels = "2.5%", pos = 4)
text(x_max, y_data[(x_data == x_max), 3], labels = "97.5%", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

# save the graph as a file
# dev.copy(jpg, file="./outputs/irf_SVEC_vars.jpg")
# dev.copy(pdf, file = "./outputs/irf_SVEC_vars.pdf")
# dev.off()

#### end #############################################
