#### R script ########################################
# Last modfied: 2019-09-09
# File name: boot_y3s1_1of2.R
# Keywords: boot: bootstrap method, confidence interval, 
#   response = y3, shock = y1,  
#   file 1 of 2, 
#   orthogonal impulse response function,
#
# Related files: (1) boot_y3s1_1of2.R, (2) boot_y3s1_2of2.R 
# 
# Written by: Hiroshi Murao 
#          
# Purpose: 
#   Perfome a bootstrap method so that we get   
#   the 3rd graph(inflation shock to interest rate) of Figure 1, 
#   which is shown in the following journal paper.
#
#   James H. Stock and Mark W. Watson (2001) "Vector Autoregressions," 
#   Journal of Economic Perspectives, Vol. 15-4, 101-115. 
#
# Data source:
#    Website of Watson: http://www.princeton.edu/~mwatson/publi.html 
#
# Data: 
#   Data type: U.S. quarterly data 	
#   Time periods of data set: 1954:01 - 2000:04
#   Time periods of estimation: 1960:01 - 2000:04 (without 4 lags)
#   Variable names:		
#     GDPD = implicit price deflator of gross domestic produc (index, 2000=100?)
#     LHUR = unemployment rate: all workers, 16 years & over (%, SA), 
#            converted from monthly data "LHUR"
#     FYFF = interest rate: Federal Fund (effective) (% per annum, NSA), 
#            converted from montly data "FYFF".
#
##########################################################
#### first things to do 
setwd("C:/book_VAR/data_works/Stock_JEP/R_prg")
library(vars) # VAR
library(urca) # unit root, cointegration
search()

#### data preparations
in_data <- read.csv("./data/data_Stock.csv")
print(in_data[1:10, ])

unemployment <- in_data$LHUR
interest <- in_data$FYFF
logP <- log(in_data$GDPD)
logP_lag1 <- c(NA, head(logP, -1))

# get annual inflation rate. 
# See Stock and Watson (2001, p.102).
inflation <- 400*(logP - logP_lag1)
inf_related <- cbind(inflation, logP, logP_lag1) 
# check calculation results
print(inf_related[1:10, ])

data_use <- data.frame(y1 = inflation, y2 = unemployment, y3 = interest)
# choose 1959, not 1960. 
# This is necessary to duplicate Stock and Watson (2001)
data_estimation <- subset(data_use, in_data$year >= 1959) 
print(data_estimation[1:10, ])
data_check <- subset(in_data, in_data$year >= 1959)
print(data_check[1:10, ])

#### set important numbers
num_y <- 3
num_z <- 1
num_lag <- 4
num_k <- num_lag*num_y + num_z
# 24 quarters = 6 years. 
# for graph, we want -1:24 = 26 periods with initial condition.  
# irf(n.ahead=25) outputs over 26 periods. 
num_h <- 25 

#### estimate VAR(4)
out_VAR <- VAR(data_estimation, p = num_lag, type = "const")

#### see what we get in out_VAR
summary(out_VAR)
# str(summary(out_VAR)) 

#### get the fitted values of y.
y_fit <- fitted.values(out_VAR)
print(y_fit[1:10, ])

#### impluse responses, using irf()
out_irf_VAR <- irf(out_VAR, response = "y3", 
n.ahead = num_h, boot = TRUE, ci = 0.95, runs = 100, orth = TRUE)

plot(out_irf_VAR)

#### get the 3rd graph of Fig. 1  
# checking how to pick up some outputs
str(summary(out_irf_VAR)) 
print(out_irf_VAR$"irf") 
print(out_irf_VAR$"irf"$"y1") 

#### get time series in outputs of irf(), in matrices.
str(summary(out_irf_VAR$irf)) 

# byR: by R, using irf() function
irf_1to3_byR <- out_irf_VAR$irf$y1
irf_2to3_byR <- out_irf_VAR$irf$y2
irf_3to3_byR <- out_irf_VAR$irf$y3

irf_band_mean <- out_irf_VAR$irf$y1
irf_band_lower <- out_irf_VAR$Lower$y1
irf_band_upper <- out_irf_VAR$Upper$y1

irf_band_1to3_byR <- cbind(irf_band_mean, irf_band_lower, irf_band_upper)
print(irf_band_1to3_byR)    

#### preparing for graph 
y_min <- min(irf_band_1to3_byR)
y_max <- max(irf_band_1to3_byR)
print(c(y_min, y_max))

# note that we set num_h = 25
x_data <- matrix(-1:24, nrow = (num_h + 1), ncol = 1)

# make sure the same nrow
print(nrow(x_data))
print(nrow(irf_band_1to3_byR))

#### plot the 3rd graph of Fig.1, with 95% confidence band
plot.new()
matplot(x_data, irf_band_1to3_byR, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dashed", "dotted", "dotdash"), 
xlim = c(-1, 24), ylim = c(-0.2, 0.8), 
xlab = "Time Horizon", ylab = "Impulse Response", 
main = "Confidence Interval: y1 to y3: use of irf()")

# save the graph as a file
# dev.copy(jpeg, file = "outputs/irf_band_1to3_byR.jpg")
# dev.off()
# make sure that we get the 3rd graph of Figure 1. p107.
 
#### get the mean curve by simulation, without irf() ##################
#### get the 3rd graph in Fig. 1 
# get coefficient estimates from out_VAR
B_whole <- Bcoef(out_VAR)
print(B_whole)
B_1 <- B_whole[,1:3]
B_2 <- B_whole[,4:6]
B_3 <- B_whole[,7:9]
B_4 <- B_whole[,10:12]
B_z <- B_whole[,13]
print(B_1)
print(B_2)
print(B_z)

#### get estimates for sigma_u
# str(summary(out_VAR))
out_summary <- summary(out_VAR)
covres <- out_summary$covres
# the following line is OK, too.
# covres <- summary(out_VAR)$covres
# print(covres)

resid <- residuals(out_VAR)
print(ncol(resid))
covar_resid <- cov(resid, y = resid)

# choose one of two options for sigma_u
print(covres)  # use this as sigma_u
print(covar_resid) # not use as sigma_u
sigma_u <- covres 

#### shock is "inflation shock". See irf(). 
select <- c(1, 0, 0)   
print(select)
scale_factor <- 1  # change if necessary

#### shock for the orthogonal impulse response function
# Cholesky facorization, lower triangular matrix 
chol_P <- t(chol(sigma_u))  
print(chol_P)

shock_orth <- chol_P%*%select*scale_factor   # (3x3)%*%(3x1) -> (3 x 1) vector
print(shock_orth)  

#### set important numbers, again
num_y <- 3
num_z <- 1
num_lag <- 4
num_k <- num_lag*num_y + num_z
# we set num_h = 25 for irf(). 
# irf(n.ahead = 25) outputs over 26 periods. for graph, we want -1:24 = 26 periods.  
num_h <- 25  
num_hwlag <- num_h + num_lag
num_hw1 <- num_h + 1
num_hw2 <- num_h + 2

#### simulation for orthogonal impulse responses
y <- matrix(0, nrow = num_y, ncol = num_hwlag)
# z <- matrix(0, nrow = num_z, ncol = num_hwlag)

# scock at h = 1 = num_lag + 1, type = orthogonal impulse
h <- num_lag + 1
y[, h] <- shock_orth

#### h-loop
for (h in (num_lag+2):num_hwlag){
y[, h] <- B_1%*%y[, h-1] + B_2%*%y[, h-2] + B_3%*%y[, h-3] + B_4%*%y[, h-4] + B_z*0
}
#### end of h-loop

print(y[, 1:10]) 

#### preparing for a graph, num_lag:num_hwlag = 26 periods
ok_y <- y[, num_lag:num_hwlag] # remove unnecessary part 

# byM: by ME, using matrices, no use irf() function
irf_1toALL_byM <- t(ok_y) # transpose for graph 
print(nrow(irf_1toALL_byM))

#### important line for final outputs
irf_1to3_byM <- irf_1toALL_byM[, 3]

y_min <- min(irf_1to3_byM)
y_max <- max(irf_1to3_byM)
print(c(y_min, y_max))

x_data <- matrix(-1:24, nrow = (num_h + 1), ncol = 1)
print(nrow(x_data))

#### plot the 3rd graph of Fig.1 in Stock and Watson (2001)
plot.new()
matplot(x_data, irf_1to3_byM, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted"), 
xlim = c(-1, 24), ylim = c(-0.2, 0.6), 
xlab = "Time Horizon", ylab = "Impulse Response", 
main = "Mean Curve of Confidence Interval: y1 to y3: no use of irf()")

#### save the graph as a file
# dev.copy(jpeg, file = "outputs/Fig1_1to3_byM.jpg")
# dev.off()

#### bootstrap method starts #############################
#### object: 95% confidence interval of irf
# make sure that the mean of residuals is zero.
resid <- residuals(out_VAR)
print(mean(resid))

#### set important numbers, again
num_y <- 3
num_z <- 1
num_lag <- 4
num_k <- num_lag*num_y + num_z
# we set num_h = 25 for irf(). 
# irf(n.ahead = 25) outputs for 26 periods.
num_h <- 25   
num_hwlag <- num_h + num_lag
num_hw1 <- num_h + 1
num_hw2 <- num_h + 2

num_T <- nrow(resid) # num_T = 164
num_Twlag <- nrow(data_estimation) # num_Twlag = 168
#num_Twlag <- num_T + num_lag

start_t <- num_lag + 1

print(c(num_T, num_Twlag))

#### matrices for bootstrap samples
ub <- matrix(0, nrow = num_T, ncol = num_y)
yb <- matrix(0, nrow = num_Twlag, ncol = num_y)

# data_estimation as a matrix
yobs <- as.matrix(data_estimation)
# print(yobs) # 188 obs

# initial values for yb
for (t in (1:num_lag)){
   yb[t,] <- yobs[t,]
}

print(yb[1:4, ])

#### set some numbers for bootstrap
num_boot <- 1000
#num_boot <- 3
# set.seed(1) # no need to use

num_boot_row <- num_y
num_boot_col <- (num_h + 1)*num_boot

start_col <- 0 # starting point for storing irf-outputs
end_col <- 0   # end point for storing irf-outputs

# a huge matrix to store all irf-values from bootstrap 
boot_irf <-  matrix(0, nrow = num_boot_row, ncol = num_boot_col)

#### test of random sampling
# random sampling
# draw_order <- sample(1:num_T, size = num_T, replace = FALSE)
# num_T = 164, num_Twlag = 168
# print(draw_order[1:10])
#
# for (t in (1:num_T)){
#  ub[t,] <- resid[draw_order[t], ]
# }
# print(cbind(draw_order, resid, ub))  # 164 obs
#### end of test 

######## boot-loop
num_count <- 0  # count the number of iterations

for (boot in (1:num_boot)){
# random sampling
draw_order <- sample(1:num_T, size = num_T, replace = TRUE)  
# num_T = 164, num_Twlag = 168
# print(draw_order[1:10])

for (t in (1:num_T)){
  ub[t,] <- resid[draw_order[t], ]
}

# print(cbind(draw_order, resid, ub))  # 164 obs

# make a bootstrap sample of y
for (t in (1:num_T)){
tt <- (t + num_lag)
yb[tt,] <- yb[tt-1,]%*%t(B_1) + yb[tt-2,]%*%t(B_2) + yb[tt-3,]%*%t(B_3) + yb[tt-4,]%*%t(B_4) + 1*t(B_z) + ub[t,]
}

# print(yb) # make sure that yb is (168 x 3)

#### estimate VAR(p), using VAR()
yb_data <- data.frame(y1 = yb[, 1], y2 = yb[, 2], y3 = yb[, 3])

boot_VAR <- VAR(yb_data, p = num_lag, type = "const")

# bootstrap-B, not use B_1, which is used as the "observed" B_1. 
BB_whole <- Bcoef(boot_VAR)
BB_1 <- BB_whole[, 1:3]
BB_2 <- BB_whole[, 4:6]
BB_3 <- BB_whole[, 7:9]
BB_4 <- BB_whole[, 10:12]
BB_z <- BB_whole[, 13]

# bootstrap-sigma_u
boot_summary <- summary(boot_VAR)
boot_sigma_u <- boot_summary$covres
# boot_sigma_u <- summary(boot_VAR)$covres  # This is OK, too.
# shock for the orthogonal impulse response function
boot_P <- t(chol(boot_sigma_u))  
boot_shock_orth <- boot_P%*%select*scale_factor   # (3x3)%*%(3x1) -> (3 x 1) vector

#### use of irf() for impluse responses 
# boot_irf <- irf(boot_VAR, response = "y3", boot=FALSE, n.ahead=num_h, orth=TRUE)
# time index "h" changes to the row-direction in outputs from irf() 
# plot(boot_irf)

#### use of simulation for impluse responses, no use of irf()
ySim <- matrix(0, nrow = num_y, ncol = num_hwlag)
# initial scock at h = 1 = (num_lag + 1)
h <- num_lag + 1
ySim[, h] <- boot_shock_orth

#### h-loop
for (h in (num_lag+2):num_hwlag){
ySim[, h] <- BB_1%*%ySim[, h-1] + BB_2%*%ySim[, h-2] + BB_3%*%ySim[, h-3] + BB_4%*%ySim[, h-4] + BB_z*0
}
# matrix "ySim" is (3 x hwlag)
#### end of h-loop

# remove unnecessary part, column at num_lag has initial condition ysim = 0 
one_boot_irf <- ySim[, num_lag:num_hwlag]

#### keep outputs into the huge matrix "boot_irf", num_hw1 = 25 + 1 = 26 
start_col <- num_count*(num_h+1) + 1
end_col <- num_count*(num_h+1) + (num_h+1)   
boot_irf[, start_col:end_col] <- one_boot_irf   # one_boot_irf is (3 x num_hout)  

num_count <- num_count + 1
}

# make sure the value of num_boot 
print(num_count)
######## end of boot-loop 

# check the content of outputs from bootstrapping 
print(boot_irf[, start_col:end_col])

#### get one of many outputs in "boot_irf"
one_irf_1to1 <- boot_irf[1, start_col:end_col]
one_irf_1to2 <- boot_irf[2, start_col:end_col]
one_irf_1to3 <- boot_irf[3, start_col:end_col]

print(one_irf_1to3)    

#### preparing for a graph 
y_min <- min(one_irf_1to3)
y_max <- max(one_irf_1to3)
print(c(y_min, y_max))

x_data <- matrix(-1:24, nrow = (num_h + 1), ncol = 1)

# make sure the same value
print(nrow(x_data))
print(length(one_irf_1to3))

#### plot a graph to see what we get in "boot_irf"
plot.new()
matplot(x_data, one_irf_1to3, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted", "dotdash"), 
xlim = c(-1, 24), ylim = c(-0.2, 0.6), 
xlab = "Time Horizon", ylab = "Impulse Response", 
main = "One of many curves by bootstrapping: y1 to y3")
# make sure that y3 is chosen

#### save files for the next step
mean_irf <- irf_1to3_byM
save(boot_irf, mean_irf, file = "./data_temp/boot_y3s1_part1.dat") 

#### remove objects from memory
rm(boot_irf, mean_irf)

###### continue to "boot_y3s1_2of2.R" ##################
#### The beginning of "boot_y3s1_2of2.R" has the following lines.
# load(file = "./data_temp/boot_y3s1_part1.dat") 
# see what we have from part 1.
# print(mean_irf)
# print(boot_irf[, 1:51]) 
#########################################################