#### R script ########################################
# Last modfied: 2019-09-09
# File name: boot_y3s1_2of2.R
# Keywords: bootstrap method, confidence interval,
#   response = y3, shock = y1,  
#   file 2 of 2, 
#   orthogonal impulse response function,
#
# Related files: (1) boot_y3s1_1of2.R, (2) boot_y3s1_2of2.R 
# 
# Written by: Hiroshi Murao 
#          
# Purpose: Perfome a bootstrap method so that  
#          we get the third graph (inflation shock to interest rate) in Fig. 1, 
#          which is shown in the following paper.
#
#    James H. Stock and Mark W. Watson (2001) "Vector Autoregressions," 
#    Journal of Economic Perspectives, Vol. 15-4, 101-115. 
#
# Data source:
#    Website of Watson: http://www.princeton.edu/~mwatson/publi.html 
#
# Data: 
#   Data type: U.S. quarterly data 	
#   Time periods of data set: 1954:01 - 2000:04
#   Time periods of estimation: 1960:01 - 2000:04 (without 4 lags)
#   Variable names:		
#     GDPD = implicit price deflator of gross domestic produc (index, 2000=100?)
#     LHUR = unemployment rate: all workers, 16 years & over (%, SA), 
#            converted from monthly data "LHUR"
#     FYFF = interest rate: Federal Fund (effective) (% per annum, NSA), 
#            converted from montly data "FYFF".
#
##########################################################
#### first things to do 
setwd("C:/book_VAR/data_works/Stock_JEP/R_prg")
library(vars) # VAR
library(urca) # unit root, cointegration
search()

#### data preparations
load(file = "./data_temp/boot_y3s1_part1.dat") 

# see what we have form part 1
print(mean_irf)
print(boot_irf[, 1:51])

irf_mean_1to3 <- mean_irf

#### set important numbers
num_boot_row <- nrow(boot_irf)
num_boot_col <- ncol(boot_irf)
# make sure (1) num_boot_row = num_y, (2) num_boot_col = num_hout*num_boot
print(c(num_boot_row, num_boot_col))

num_y <- 3
num_z <- 1
num_lag <- 4
num_k <- num_lag*num_y + num_z

# we set num_h = 25. irf(n.ahead = 25) outputs over 26 periods. 
# for graph, we want -1:24 = 26 periods with y = 0 at h = -1.  
num_h <- 25  
num_hwlag <- num_h + num_lag
num_hw1 <- num_h + 1
num_hw2 <- num_h + 2

num_boot <- as.integer(num_boot_col/(num_h + 1))
print(num_boot)
# num_boot <- 1000

#### prepare for a graph
irf_mean_1to3_graph <- irf_mean_1to3[1:(num_h + 1)]   # with y = 0
# irf_mean_1to3_graph <- irf_mean_1to3[2:(num_h + 1)] # without y = 0

y_min <- min(irf_mean_1to3_graph)
y_max <- max(irf_mean_1to3_graph)
print(c(y_min, y_max))

# we set num_h = 25
x_data <- matrix(-1:24, nrow = (num_h + 1), ncol = 1)

# make sure that num_row = 26 for the graph
print(nrow(x_data))
print(length(irf_mean_1to3_graph))

#### plot the mean curve, 3rd graph of Fig.1 in Stock and Watson (2001)
plot.new()
matplot(x_data, irf_mean_1to3_graph, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted"), 
xlim = c(-1, 24), 
ylim = c(-0.2, 0.6), 
xlab = "Time Horizon", 
ylab = "Impulse Response", 
main = "Mean Curve of Confidence Interval: y1 to y3")

#### get a bootstrap distribution at each h, y1 to y3
boot_irf_1to3 <- boot_irf[3, ]  # boot_irf_1to3 is (1 x 25,000)
# make sure that zero values appear in every interval of (num_h+1) 
print(boot_irf_1to3[1:51]) 

#### make matrices for data stansformation
table_NH <- matrix(0, nrow = num_boot, ncol = (num_h + 1))
sorted_table_NH <- matrix(0, nrow = num_boot, ncol = (num_h + 1))
mover <- matrix(0, nrow = num_boot, ncol = 1)

#### boot-loop
first_pt <- 0
last_pt <- 0

for (boot in (1:num_boot)){  
  first_pt <- (num_h + 1)*(boot - 1) + 1     
  last_pt <- (num_h + 1)*boot 
  table_NH[boot, ] <- boot_irf_1to3[first_pt:last_pt]
}
#### end boot-loop

# check the above oparation, the 1st column is zero values in table_NH[, 1]
print(c(first_pt, last_pt))

print(table_NH[1:30, 1:5])

#### sort the data by colunm (i.e., each h)
for (h in (2:(num_h + 1))){ 
 order_list <- order(table_NH[, h])
 mover <- table_NH[order_list, h]
 sorted_table_NH[, h] <- mover
}

# make sure the above oparation
print(order_list[1:10])
print(sorted_table_NH[1:50, 1:5])

##### Efronfs percentile 
lo_n <- as.integer(0.025*num_boot)
up_n <- as.integer(0.975*num_boot)
print(c(lo_n, up_n))

irf_lo_1to3 <- sorted_table_NH[lo_n, ]
irf_up_1to3 <- sorted_table_NH[up_n, ]

print(irf_lo_1to3)
print(irf_up_1to3)

lo_min <- min(irf_lo_1to3)
lo_max <- max(irf_lo_1to3)
up_min <- min(irf_up_1to3)
up_max <- max(irf_up_1to3)

print(c(lo_min, lo_max, up_min, up_max))

efron_band_1to3 <- t(rbind(irf_mean_1to3, irf_lo_1to3, irf_up_1to3))
print(efron_band_1to3)

#### prepare for a graph
efron_band_1to3_graph <- efron_band_1to3[1:num_hw1, ] # with y = 0
# efron_band_1to3_graph <- efron_band_1to3[2:num_hw1, ]  # without y = 0
print(efron_band_1to3_graph)

y_min <- min(efron_band_1to3_graph)
y_max <- max(efron_band_1to3_graph)
print(c(y_min, y_max))

# we set num_h = 25. 
x_data <- matrix(-1:24, nrow = (num_h + 1), ncol = 1)

# make sure that num_row = 26 for the graph
print(nrow(x_data))
print(nrow(efron_band_1to3_graph))

#### plot Efron's band, 3rd graph of Fig.1 in Stock and Watson (2001)
plot.new()
matplot(x_data, efron_band_1to3_graph, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dashed"), 
xlim = c(-1, 29), 
ylim=c(-0.2, 0.8), 
xlab = "Time Horizon", 
ylab = "Impulse Response", 
main = "Efronfs Confidence Interval: y1 to y3")

# add curve lables
abline(h = 0, col = "gray")
text(24, efron_band_1to3_graph[x_data == 24, 1], labels = "mean", pos = 4)
text(24, efron_band_1to3_graph[x_data == 24, 2], labels = "2.5%", pos = 4)
text(24, efron_band_1to3_graph[x_data == 24, 3], labels = "97.5%", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

#### save the graph as a file
# dev.copy(jpeg, file = "./outputs/Efron_band_1to3_byM.jpg")
# dev.off() 

#### Hallfs percentile method #########################
# make matrices for Hallfs percentile method
d_table_NH <- matrix(0, nrow = num_boot, ncol = (num_h + 1))
d2_table_NH <- matrix(0, nrow = num_boot, ncol = (num_h + 1))
mean_table_H <- matrix(0, nrow = 1, ncol = (num_h + 1))
 
## double loops
for (h in (2:(num_h + 1))){
  mean_table_H[h] <- mean(table_NH[, h])
  for (boot in (1:num_boot)){ 
    d_table_NH[boot, h] <- sorted_table_NH[boot, h] - irf_mean_1to3[h]   
    d2_table_NH[boot, h] <- sorted_table_NH[boot, h] - mean_table_H[h] 
  } # end of boot-loop
} # end of h-loop
## end of double loops

# make sure the above oparation, use d_table, not d2_table
print(d_table_NH[1:10, ])

## Hallfs percentile 
lo_n <- as.integer(0.025*num_boot)
up_n <- as.integer(0.975*num_boot)
print(c(lo_n, up_n))

d_alph <- d_table_NH[lo_n, ]
d_1_alph <- d_table_NH[up_n, ]

print(d_alph)
print(d_1_alph)

irf_lo_1to3 <- irf_mean_1to3 - d_1_alph
irf_up_1to3 <- irf_mean_1to3 - d_alph

# check what we get
lo_min <- min(irf_lo_1to3)
lo_max <- max(irf_lo_1to3)
up_min <- min(irf_up_1to3)
up_max <- max(irf_up_1to3)

print(c(lo_min, lo_max, up_min, up_max))

hall_band_1to3 <- t(rbind(irf_mean_1to3, irf_lo_1to3, irf_up_1to3))
print(hall_band_1to3)

## prepare for a graph
hall_band_1to3_graph <- hall_band_1to3[1:num_hw1, ] # with y=0
# hall_band_1to3_graph <- hall_band_1to3[2:num_hw1, ]  # without y=0
print(hall_band_1to3_graph)

y_min <- min(hall_band_1to3_graph)
y_max <- max(hall_band_1to3_graph)
print(c(y_min, y_max))

# we set num_h = 25. 
x_data <- matrix(-1:24, nrow=(num_h + 1), ncol = 1)

# make sure that num_row = 26 for the graph
print(nrow(x_data))
print(nrow(hall_band_1to3_graph))

#### plot Hall's band, 3rd graph of Fig.1 in Stock and Watson (2001)
plot.new()
matplot(x_data, hall_band_1to3_graph, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dashed"), 
xlim = c(-1, 29), 
ylim = c(-0.2, 0.8), 
xlab = "Time Horizon", 
ylab = "Impulse Response", 
main = "Hall's Confidence Interval: y1 to y3")

# add curve lables
abline(h = 0, col = "gray")
text(24, hall_band_1to3_graph[x_data == 24, 1], labels = "mean", pos = 4)
text(24, hall_band_1to3_graph[x_data == 24, 2], labels = "2.5%", pos = 4)
text(24, hall_band_1to3_graph[x_data == 24, 3], labels = "97.5%", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

#### save the graph as a file.
# dev.copy(jpeg, file="./outputs/Hall_band_1to3_byM.jpg")
# dev.off()

#### end #############################################
