#### R script ########################################
# Last modfied: 2019-09-27
# File name: fevd_two_methods_VAR.R
# Keywords: forecast error variance decompotition, 
#    two methods: (1) use of fevd() function, (2) use of matrix operations,
#    based on a VAR model, 
#
# Written by: Hiroshi Murao
#
# Purpose: 
#   Perfome the forecast error variance decompotition
#   so that we get "Table 1 Biii" in the following journal paper.
#
#   James H. Stock and Mark W. Watson (2001) "Vector Autoregressions," 
#   Journal of Economic Perspectives, Vol. 15-4, 101-115. 
#
# Data source:
#    Website of Watson: http://www.princeton.edu/~mwatson/publi.html 
#
# Data: 
#   Data type: U.S. quarterly data 	
#   Time periods of data set: 1954:01 - 2000:04
#   Time periods of estimation: 1960:01 - 2000:04 (without 4 lags)
#   Variable names:		
#     GDPD = implicit price deflator for gross domestic product,  
#            the chain-weighted GDP price index (base year: 2000=100?)          
#     LHUR = unemployment rate: all workers, 16 years & over (%, SA), 
#            converted from monthly data.
#     FYFF = interest rate: Federal Fund (effective) (% per annum, NSA), 
#            converted from montly data.
#
##########################################################
#### first things to do 
setwd("C:/book_VAR/data_works/Stock_JEP/R_prg")
library(vars) # VAR
library(urca) # unit root, cointegration
search()

#### data preparations
in_data <- read.csv("./data/data_Stock.csv")
print(in_data)

unemployment <- in_data$LHUR
interest <- in_data$FYFF
logP <- log(in_data$GDPD)
logP_lag1 <- c(NA, head(logP, -1))

# get annual inflation rate. See Stock and Watson (2001, p.102).
inflation <- 400*(logP - logP_lag1)  
print(cbind(inflation, logP, logP_lag1))

data_pick <- data.frame(y1 = inflation, y2 = unemployment, y3 = interest)
# choose 1959, not 1960. This is necessary to duplicate Table 1 Biii.
y_data_VAR <- subset(data_pick, in_data$year >= 1959)  
print(y_data_VAR)

#### set important numbers
num_y <- 3
num_z <- 1
num_lag <- 4
num_x <- num_lag*num_y + num_z
num_h <- 25 # horizon for irf()

#### estimate a VAR(4), using VAR()
out_VAR <- VAR(y_data_VAR, p = num_lag, type = "const")

#### see what we get in out_VAR
summary(out_VAR)

#### impluse responses, using irf()
out_irf_orth <- irf(out_VAR, n.ahead = num_h, 
boot = TRUE, ci = 0.95, runs = 100, orth = TRUE)
plot(out_irf_orth)

#### forecast error variance decomposition, using fevd() 
out_fevd_byR <- fevd(out_VAR, n.ahead = 12)

#### get Table 1 Biii in Stock and Watson (2001, p.105), using fevd() 
str(summary(out_fevd_byR))
print(out_fevd_byR$"y3")  

#### preparing for graph 
y_data_byR <- out_fevd_byR$"y3"

y_min <- min(y_data_byR)
y_max <- max(y_data_byR)
print(c(y_min, y_max))

x_data <- matrix(1:12, nrow = 12, ncol = 1)
x_min <- min(x_data)
x_max <- max(x_data)
print(c(x_min, x_max))

print(nrow(x_data))
print(nrow(y_data_byR))

#### plot a graph, which is equal to Table 1@Biii.
plot.new()
matplot(x_data, y_data_byR, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted", "dotdash"), 
xlim = c(1, 14), 
ylim = c(-0.02, 1.00), 
xlab = "forecast horizon (quarter)", 
ylab = "relative variance contribution (RVC)", 
main = "Variance Decomposition: use of fevd()")

## add curve lables and the like
abline(h = 0, col = "gray")
text(x_max, y_data_byR[(x_data == x_max), 1], labels = "y1", pos = 4)
text(x_max, y_data_byR[(x_data == x_max), 2], labels = "y2", pos = 4)
text(x_max, y_data_byR[(x_data == x_max), 3], labels = "y3", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

## save the graph as a file
dev.copy(jpeg, file = "./outputs/fevd_byR.jpg")
dev.off()

#### main part of this program ############################ 
#### get the same results without using fevd()
# get coefficient estimates from out_VAR
B_whole <- Bcoef(out_VAR)
print(B_whole)
B_1 <- B_whole[,1:3]
B_2 <- B_whole[,4:6]
B_3 <- B_whole[,7:9]
B_4 <- B_whole[,10:12]
B_z <- B_whole[,13]
print(B_1)
print(B_z)

# get estimates for sigma_u
str(summary(out_VAR))
out_summary <- summary(out_VAR)
covres <- out_summary$covres
print(covres)  # use this as sigma_u

resid <- residuals(out_VAR)
covar_resid <- cov(resid, y=resid)
print(covar_resid) # not use as sigma_u

sigma_u <- covres 

# shock is "inflation shock". Seee irf(). 
select <- c(1, 0, 0)   
print(select)
scale_factor <- 1  # change if necessary

#### shock for the orthogonal impulse response function
# Cholesky facorization, lower triangular matrix 
P <- t(chol(sigma_u))  
print(P)

shock_orth <- P%*%select*scale_factor   # (3x3)%*%(3x1) -> 3x1 vector
print(shock_orth)  

#### set important numbers
num_y <- 3
num_z <- 1
num_lag <- 4
num_x <- num_lag*num_y + num_z
num_h <- 12  # horizon for fevd. See Stock and Watson.
num_col <- num_h + num_lag

#### make matrices for RVC
y <- matrix(0, nrow=num_y, ncol=num_col)
z <- matrix(0, nrow=num_z, ncol=num_col)
V <- matrix(0, nrow=num_y, ncol=num_col)
V_s1 <- matrix(0, nrow=num_y, ncol=num_col)
V_s2 <- matrix(0, nrow=num_y, ncol=num_col)
V_s3 <- matrix(0, nrow=num_y, ncol=num_col)

#### checking computations before two loops
h <- num_lag + 1
y[, h-1] <- shock_orth
V[, h] <- V[, h-1] + y[,h-1]^2 
print(y[, (h-2):(h+2)])
print(V[, (h-2):(h+2)])

#### loop for i = 1,2,---,num_y
for (i in (1:num_y)){
  {if (i == 1){select <- c(1, 0, 0)}
  else if (i == 2){select <- c(0, 1, 0)}
  else {select <- c(0, 0, 1)}
  } # end of if 
 
  shock_orth <- P%*%select*scale_factor   # (3x3)%*%(3x1) -> 3x1 vector
  h <- num_lag + 1
  y[, h-1] <- shock_orth
  V[, h] <- V[, h-1] + y[,h-1]^2 

  for (h in (num_lag+2):num_col){
     y[, h-1] <- B_1%*%y[, h-2] + B_2%*%y[, h-3] + B_3%*%y[, h-4] + B_4%*%y[, h-5] # + B_z%*%z[, h-1]
     V[, h] <- V[, h-1] + y[,h-1]^2
     } # end of if
     {if (i == 1) {V_s1 <- V} 
     else if (i == 2) {V_s2 <- V} 
     else {V_s3 <- V}
  } # end of h-loop
} # end of i-loop
#### end of two loops

# checking computations after the two loops
print(y)
print(V)
print(V_s1) 
print(V_s2)
print(V_s3)

#### preparing for RVC 
sum_V <- matrix(0, nrow=num_y, ncol=num_col)
RVC_s1 <- matrix(0, nrow=num_y, ncol=num_col)
RVC_s2 <- matrix(0, nrow=num_y, ncol=num_col)
RVC_s3 <- matrix(0, nrow=num_y, ncol=num_col)

#### compute RVC
sum_V <- V_s1 + V_s2 + V_s3
RVC_s1 = V_s1/sum_V
RVC_s2 = V_s2/sum_V
RVC_s3 = V_s3/sum_V

print(RVC_s3)

#### preparing for Table 1 Biii in Stock and Watson (2001, p.105)
RVC_r1 <- matrix(0, nrow=num_y, ncol=num_col)
RVC_r2 <- matrix(0, nrow=num_y, ncol=num_col)
RVC_r3 <- matrix(0, nrow=num_y, ncol=num_col) 

# response = y3  
RVC_r3[1, ] <- RVC_s1[3, ] 
RVC_r3[2, ] <- RVC_s2[3, ]
RVC_r3[3, ] <- RVC_s3[3, ]

out_RVC_r3 <- RVC_r3[, (num_lag+1):num_col]
print(out_RVC_r3)

#### preparing for a graph
y_data <- t(out_RVC_r3)

y_min <- min(y_data)
y_max <- max(y_data)
print(c(y_min, y_max))

x_data <- matrix(1:12, nrow = 12, ncol = 1)
x_min <- min(x_data)
x_max <- max(x_data)
print(c(x_min, x_max))

print(nrow(x_data))
print(nrow(y_data))

#### plot another graph, 
# which is equal to Table 1@Biii.
plot.new()
matplot(x_data, y_data, 
type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed", "dotted", "dotdash"), 
xlim = c(1, 14), 
ylim = c(-0.02, 1.00), 
xlab = "forecast horizon (quarter)", 
ylab = "relative variance contribution (RVC)", 
main = "Variance Decomposition of Interest Rate")
# main = "Variance Decomposition: no use of fevd()")

## additional information to the graph
abline(h = 0, col = "gray")
text(x_max, y_data[(x_data == x_max), 1], labels = "y1", pos = 4)
text(x_max, y_data[(x_data == x_max), 2], labels = "y2", pos = 4)
text(x_max, y_data[(x_data == x_max), 3], labels = "y3", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

## save the graph as a file
# dev.copy(jpeg, file = "./outputs/fevd_byM.jpg")
# dev.off()

#### make "Table Biii" in Stock and Watson (2001).
# compute root_MSE_y3 
out_sum_V <- sum_V[,(num_lag+1):num_col]
root_MSE <- sqrt(out_sum_V)
root_MSE_y3 <- as.matrix(root_MSE[3, ])
print(sum_V)
print(root_MSE)
print(root_MSE_y3)

# make a table looks like "Table Biii"
table_fevd_y3 <- data.frame(h = x_data, 
standard_error = root_MSE_y3, 
RVC_31 = y_data[, 1],
RVC_32 = y_data[, 2], 
RVC_33 = y_data[, 3])

print(table_fevd_y3)

# print "Table Biii"
print(table_fevd_y3[c(1,4,8,12), ])

#### end #############################################
