#### R script ##########################################################
# Last modfied: 2021-12-13
# File name: hd_VMA_v1.R
# Keywords: historical decomposition based on the VMA representation 
#     of a stationary VAR(p) process,
#     Version "VMA_v1",
#     use the codes equivalent to Phi() function in Package "vars",
#     use the VAR(1) representation of VAR(p) process,
#     some codes are simillar to the those of hd() function in "svars",    
#
# Purpose: perform the historical decomposition based on SVAR(p), 
#     using the author's idea of how to manually conduct. 
#
# Written by: Hiroshi Murao
#
# Journal paper related to data source:
#   Stock, James H., and Mark W. Watson (2001) "Vector Autoregressions", 
#   Journal of Economic Perspectives, Vol.15, No.4, pp.101-115. 
#
# Data source:
#    Website of Watson: http://www.princeton.edu/~mwatson/publi.html 
#
# Data information: 
#   Data type: U.S. quarterly data 	
#   Time periods of original data set: 1954:01 - 2000:04
#   Time periods of for use: 1959:01 - 2000:04 (with 4 lags)
#   Sample time periods of estimation: 1960:01 - 2000:04 (without 4 lags)
# 
# List of variables:		
#     GDPD = implicit price deflator of gross domestic produc (index, 2000 = 100?)
#     LHUR = unemployment rate: all workers, 16 years & over (percent, SA), 
#            converted from monthly data "LHUR"
#     FYFF = interest rate: Federal Fund (effective) (percent per annum, NSA), 
#            converted from montly data "FYFF".
#
###############################################################################
#### first things to do 
setwd("C:/book_VAR/data_works/Stock_JEP/R_prg")
library(vars) # VAR
library(svars) # SVAR 
search()

#### data preparations
in_data <- read.csv("./data/data_Stock.csv")
print(in_data[1:10, ])

unemployment <- in_data$LHUR
interest <- in_data$FYFF
logP <- log(in_data$GDPD)
logP_lag1 <- c(NA, head(logP, -1))

# get annual inflation rate. 
# See Stock and Watson (2001, p.102).
inflation <- 400*(logP - logP_lag1)  
print(cbind(inflation, logP, logP_lag1)[1:5, ])

#### data_pick
data_pick <- data.frame(year = in_data$year, quarter = in_data$quarter,
    inflation = inflation, unemployment = unemployment, interest = interest)

print(data_pick[1:5, ])

# data_pick_noNA <- as.numeric(na.omit(data_pick))
data_pick_noNA <- na.omit(data_pick)
print(data_pick_noNA[1:5, ])

#### data for use
# choose year >= 1959. 
data_use <- subset(data_pick, data_pick$year >= 1959)  
print(data_use[1:10, ])

print("The first year of the data set for use")
print(data_use[1, 1])

print("The first quarter of the data set for use")
print(data_use[1, 2])

#### time: 1 = 1959Q1

#### data in difference
# after the selection of time periods
inflation_diff <- diff(data_use$inflation, lag = 1)
unemployment_diff <- diff(data_use$unemployment, lag = 1)
interest_diff <- diff(data_use$interest, lag = 1)

print(inflation_diff[1:5])

data_diff_pick <- cbind(inflation_diff, unemployment_diff, interest_diff)

print(data_diff_pick[1:5, ]) 

num_data_diff <- nrow(data_diff_pick)
print(num_data_diff)
# nrow(data_diff_pick) = 167 

#### data_diff for use
data_diff_use <- data_diff_pick
# print(data_diff_use)

#### data for VAR(p) or the like
ys_level <- data.frame(y1 = data_use$inflation, 
    y2 = data_use$unemployment, 
    y3 = data_use$interest)

print(nrow(ys_level))
# nrow(ys_level) = 168

ys_diff <- data.frame(y1_diff = inflation_diff, 
     y2_diff = unemployment_diff, 
     y3_diff = interest_diff)

print(ys_diff[1:5, ])
print(nrow(ys_diff))
# nrow(ys_diff) = 167

x1 <- subset(logP_lag1, in_data$year >= 1959)
x1sq <- x1^2
exogen_data <- data.frame(x1 = x1, x2 = x1sq)
print(exogen_data[1:5, ])

#### Estimation ########################################################
# and tasks after the estimation

#### estimate VAR(p), using VAR() in "vars"
# we use ys_level
# we use "const"
num_lag <- 4
out_VAR <- VAR(y = ys_level, p = num_lag, type = "const", exogen = NULL)
summary(out_VAR)

##### set important numbers
num_obs <- nrow(ys_level)
num_T <- num_obs - num_lag
num_y <- 3
num_yLag <- num_y*num_lag
num_z <- 1
num_para <- num_yLag + num_z

#### get coefficient estimates from out_VAR
B_VAR <- Bcoef(out_VAR)
print(B_VAR)
# B_1_VAR <- B_VAR[, 1:3]
# B_2_VAR <- B_VAR[, 4:6]
# B_3_VAR <- B_VAR[, 7:9]
# B_4_VAR <- B_VAR[, 10:12]
# B_z_VAR <- B_VAR[, 13]
# change if necessary
# B_z_VAR <- B_VAR[, 13:14]
# print(B_1_VAR)
# print(B_z_VAR)

# B for "big" representation
B_lags <- B_VAR[, 1:12]
print(B_lags)

#### estimate SVAR(p), using SVAR() function in Package "vars"
# num_y = 3 => n(n-1)/2 = 3 => need 3 restrictions
# We initially set the impact matrix "Amat" to be a lower trianguar matrix.

## make Amat
Amat <- diag(3)
Amat[2, 1] <- NA
Amat[3, 1:2] <- NA
print(Amat)
# Amat is a lower trainguar matrix 
# Change if necessary

## make Bmat
Bmat <- diag(3)
Bmat[1, 1] <- NA
Bmat[2, 2] <- NA
Bmat[3, 3] <- NA
print(Bmat)
# Bmat is usually the diagonal matrix

#### estimate SVAR(p), using SVAR() in "vars"
out_SVAR <- SVAR(x = out_VAR, 
    Amat = Amat, Bmat = Bmat,
    estmethod = "scoring", 
    lrtest = FALSE, max.iter = 100, cov.crit = 0.1e-6, maxls = 1.0)
     
# The following line is used for estmethod = "direct".
# out_SVAR <- SVAR(x = out_VAR, 
#    Amat = Amat, Bmat = Bmat,
#    estmethod = "direct", lrtest = FALSE)

summary(out_SVAR)

#### get some matrices from out_SVAR
names(out_SVAR)
A_SVAR <- out_SVAR$A
B_SVAR <- out_SVAR$B

#### one way to get an option for impact matrix "Cmat",
# using SVAR() function in "vars".
invA_SVAR <- solve(A_SVAR)
C_SVAR <- invA_SVAR%*%B_SVAR
print(C_SVAR)

#### annother way to get an option for impact matrix "Cmat", 
# using Cholesky facorization,

# one way to get sigma_u
out_summary <- summary(out_VAR) 
print(out_summary)
print(out_summary$covres)
sigma_covres <- out_summary$covres

# another way to get simga_u
u_hat <- residuals(out_VAR)
num_T <- nrow(u_hat)
print(num_T)
# num_T = 164
print(u_hat[1:5, ])
sigma_resid <- crossprod(u_hat)/(num_T - num_para)

# compare two options for sigma_u
print(sigma_covres)
print(sigma_resid)

# choose sigma_u
sigma_u <- sigma_covres
print(sigma_u)  # this is covariance matrix

# Cholesky facorization, 
# a lower triangular matrix 
chol_P <- t(chol(sigma_u))  
print(chol_P)

#### another way to get an option for impact metrix "Cmat", 
# using one of id() functions in "svars", 
out_id <- id.dc(x = out_VAR)

summary(out_id)
names(out_id)
# The option for "Cmat" is called as "B" in out_id.
C_id <- out_id$B

#### compare the three options for the impact matrix "Cmat": 
# (1) "C_SVAR" from SVAR() in "vars"
# (2) "chol_P" from VAR() in "vars"
# (3) "C_id" from id.dc() in "svars"

print(C_SVAR)
print(chol_P)
print(C_id)

# C_SVAR = chol_P if the SVAR(p) uses a lower triangular matrix for "Amat".
# C_SVAR and chol_P are different for other cases.

#### information about "Cmat"
# (1) Cmat <- solve(A) %*% B. This is carried out if the AB type of SVAR is used
# (2) Cmat <- B.  This is carried out if the B type of SVAR is used. 
#                 The manual of hd() shows the B type SVAR.
# (3) Cmat <- chol_P. This choise is pupular if VAR is used. 

#### Choose the impact matrix "Cmat" suitable to the purpose.
# We want to duplicate the output of fd() function in "svars.
Cmat <- C_id
# Cmat <- chol_P
# Cmat <- C_SVAR
print(Cmat)

#### Phi part #################################################################
# equivalent to Phi() function in "vars".
# We call it as "Phi-equivalent".
#
# Notes for Phi() function:
# out_Phi <- Phi(x = out_SVAR, nstep = num_T)
# out_Phi[i, j, h], where 
#  i: response variable number, 
#  j: shock variable number, 
#  h: time index number (horizon).
# number of outputs = num_T + 1, due to the inclusion of Phi(0) = Cmat.
#
# Note that the matrix "Phi" is the coefficient matrix of innovations. 
# The matrix "Phi" is often denoted as "Psi" or "Theta" in theoretical books.  
# The matrix "Phi" might be called as "innovMat", "ysAll", or the like.

#### get important numbers for "Phi-equivalent"
# num_T <- ncol(u_hat)
num_T <- nrow(u_hat)
print(num_T)

#### prepare for the VAR(1) representation of VAR(p) process
# We call it as "big".

# make the "big" version of B_lags
B_big <- matrix(0, nrow = num_yLag, ncol = num_yLag)
diagLow <- diag(num_y*(num_lag - 1)) 

B_big[((num_y + 1):num_yLag), 1:((num_lag - 1)*num_y)] <- diagLow 

# make sure the content of B_big
print(B_big)

# make sure the content of B_big, again
B_big[1:num_y, 1:num_yLag] <- B_lags
print(B_big)

#### make an array for outputs, and so on. 
num_Phi <- num_T
out_Phi <- array(0, c(num_y, num_y, num_Phi))

# note: Phi(0) = Cmat
out_Phi[, , 1] <- Cmat

Phi_big <- diag(num_yLag)

# set ending point of s-loop
s_end <- num_Phi - 1

#### loop for "Phi-equivalent"
# make the innovation coefficient array called "out_Phi"
for (s in 1:s_end) {
  # The following line is equivalent to: Phi_big(s) = B_big %*% Phi_big(s-1)
  Phi_big <- B_big%*%Phi_big
  # get a part of the "big" system for ouput.
  out_Phi[, , (s + 1)] <- Phi_big[1:num_y, 1:num_y]%*%Cmat
  } 
#### end of s-loop

#### end of "Phi-equivalent" #####

#### check the content of "out_Phi" 
print(out_Phi[, , 1])
print(Cmat)
# note: out_Phi[, , 1] = Phi(0) = Cmat.

# test print for j = 2
print(out_Phi[, 2, 1])
print(out_Phi[, , 1])

# test print for j = 3
print(out_Phi[, 3, 1])
print(out_Phi[, , 1])

#### prepare for the VAR(1) representation of VAR(p) process ###########################
# We call it as "big".

# make the "big" version of Cmat
# note: B_big is (num_yLag x num_yLag), B_big %*% y_big is (num_yLag x 1).
C_big <- matrix(0, nrow = num_yLag, ncol = num_y)

C_big[1:num_y, ] <- Cmat

# check the content of C_big 
print(C_big)
print(Cmat)
# C_big looks all right.

#### make "selectFirst" for "big"
# This matrix is used to take out a part of "y_big"
selectFirst <- matrix(0, nrow = num_y, ncol = num_yLag)
selectFirst[1:num_y, 1:num_y] <- diag(num_y)

print(selectFirst)

print(nrow(selectFirst))
print(ncol(selectFirst))
# 3 x 12

#### Core part of historical decomposition ##################################
# some codes are simmilar to those of hd() function in "svars",

##### set important numbers for the historical decomposition
dim(out_Phi)
# 3 x 3 x 165, 
# See the manual for Phi() function.

num_Phi <- dim(out_Phi)[3]
print(num_Phi)

# the following "num_obs" is different from "num_lag + num_T",
# due to the use of "big" system,
# the following numbers might be different if 
# we use SVAP(p) process, where num_obs = num_lag + num_T.

num_obs <- num_Phi
num_T <- num_obs
# note: num_T = num_obs - num_lag if we use the usual VAR(p) presentation.

### get "w" data for structual shocks 
invC <- solve(Cmat) 
# invC <- MASS::ginv(Cmat)
is.matrix(invC)
print(invC)

w_hat <- as.matrix(invC%*%t(u_hat))
is.matrix(w_hat)
# w_hat is (3 x 164)
print(w_hat[, 1:5])

w <- matrix(0, nrow = num_y, ncol = num_obs)
w_end <- num_obs
w[, 1:w_end] <- w_hat
print(w[, 1:5])
 
#### make an array for loop-computation 
y_array <- array(0, dim = c(num_y, num_y, num_T)) 

#### set starting and ending points of t-loop
# We use "t" index for loop-computation.
# We do not use "h" index for loop-computation.

t_start <- 1
t_end <- num_obs
print(c(t_start, t_end))

#### loop section for historical decomposition
# fix (i, j) before the computation at h,
# where the resulting value will be a scalor,
for (i in 1:num_y) {
for (j in 1:num_y) {
  # select_s <- matrix(0, nrow = num_y, ncol = num_y)
  # select_s[j, j] <- 1
  # wj <- select_s%*%w
  # The following line is OK since the scalor computation is carried out in an "array".
  wj <- w[j, ] # dimension: (1 x num_obs)

  for (t in t_start:t_end) {
    # note(1): out_Psi[ , , 1] = Psi(0) = Cmat
    # note(2): The time index "t" for the Psi matrix changes from "0" to "t-1" in theoretical books. 
    # note(3): dimesion: (K x 1) <- (K x t) %*% (t x 1) # => error message.
    # note(4): dimesion: (1 x 1) <- (1 x t) %*% (t x 1) # => OK, and use "t(wj)[t:1]", 
    y_array[i, j, t] <- out_Phi[i, j, 1:t]%*%t(wj)[t:1]    
  } # end of t-loop 
      
} # end of j-loop
} # end of i-loop
#### end of loop-section

#### remove unnecessary part in y_array[, , ,]
dim(y_array)

# There is no unnecessary part for this VMA version.
# y_array <- y_array[, , -t_end]
# dim(y_array)

print(num_T)

### see what we get from the loop-computation
print(w[, 1:5])
print(wj[1:5])

dim(y_array)
# 3 x 3 x 164
print(y_array[, 1, 1:3])

# check some values
print(y_array[, 1, 1:3])
print(y_array[3, 1, 1:5])

#### get outputs as vectors 
# set response = i = "y3" 
# get outputs from y_array[3, , ] 

y3_s1 <- as.vector(y_array[3, 1, ])
print(y3_s1[1:3])
print(y_array[, , 1:3])

y3_s2 <- as.vector(y_array[3, 2, ])
print(y3_s2[1:3])
print(y_array[, , 1:3])
   
y3_s3 <- as.vector(y_array[3, 3, ])
print(y3_s3[1:3])
print(y_array[, , 1:3])

### get the total effect value of shocks = (s1, s2, s3) on response = y3  
# one way to get "y3_tot"
y3_tot <- y3_s1 + y3_s2 + y3_s3
print(y3_tot[1:5])
print(length(y3_tot))

# anothe way to get "y3_tot"
y3_Sums <- colSums(y_array[3, , ])
print(y3_Sums[1:5])
print(y3_tot[1:5])
# "y3_Sums" and "y3_tot" should be equal.

#### get "demeaned" value 
# note: response variable number = series number = i = 3
y3_raw <- ys_level[-c(1:num_lag), 3]
print(y3_raw[1:5])

mean_y3 <- mean(y3_raw)

y3_dm <- matrix(0, nrow = num_T, ncol = 1)

# t-loop for "demeaned" value "y3_dm"
for (t in 1:num_T){ 
   y3_dm[t] <- y3_raw[t] - mean_y3
} # end of t-loop 

# comapre some values
print(y3_dm[1:10])
print(y3_tot[1:10])

#### make final outputs for graphic presentations 
# Take care of the case of "num_tran > 1".
# H = H1 + H2, H = T,
# H1 = num_tran

num_tran <- 0 
# num_tran <- 30 

# num_tran <- floor(num_obs*transition)
# "transition" is a value in [0,1].
# See "transition" in hd() function in "svars".

# max horizon for graphic presentations
num_H2 <- num_obs - num_lag - num_tran 

print(c(num_lag, num_tran, num_H2, num_T, num_obs))

# outputs for graphic presentations
if (num_tran >= 2){
  y3_dm <- y3_dm[-c(1:num_tran)]
  y3_tot <- y3_tot[-c(1:num_tran)]
  y3_s1 <- y3_s1[-c(1:num_tran)]
  y3_s2 <- y3_s2[-c(1:num_tran)]
  y3_s3 <- y3_s3[-c(1:num_tran)]
} else if (num_tran == 1) {
  y3_dm <- y3_dm[-1]
  y3_tot <- y3_tot[-1]
  y3_s1 <- y3_s1[-1]
  y3_s2 <- y3_s2[-1]
  y3_s3 <- y3_s3[-1]
} # end of if-section
 
# check some values
print(y3_dm[1:5])
print(y3_tot[1:5])
print(y3_s1[1:5])
print(y3_s2[1:5])
print(y3_s3[1:5])

#### Plot (1) ##############################################################
# plot for "dm" vs. "tot"
# "dm": Demeaned y3, 
# "tot": Total of (y3_s1 + y3_s2 + ---).
# get y_data for "3_tot"
y_data_3_tot <- cbind(y3_dm, y3_tot) 

print(y_data_3_tot[1:10, ])

y_min <- min(y_data_3_tot)
y_max <- max(y_data_3_tot)
print(c(y_min, y_max))

## get x_data
num_row <- nrow(y_data_3_tot)
print(num_row)
x_data <- matrix(1: num_row, nrow = num_row, ncol = 1)
print(x_data[1:10])
x_min <- min(x_data)
x_max <- max(x_data)
print(c(x_min, x_max))

#### plot for "3_tot"
plot.new()
matplot(x_data, y_data_3_tot, 
type = "l", lwd = 2, col = 1, 
lty = c("dotted", "solid", "dashed"), 
xlim = c(1, (x_max + 40)), 
ylim = c(-5.0, 12.0), 
xlab = "time (quarter) (1 = 1960Q1)", 
ylab = "value (percent)",
main = "dm vs. tot, response = y3, Array v1")

#### additional information to the graph
abline(h = 0, col = "gray")
text(x_max, y_data_3_tot[x_data == x_max, 1], labels = "HD_3_dm", pos = 4)
text(x_max, y_data_3_tot[x_data == x_max, 2], labels = "HD_3_tot", pos = 4)
# pos = 1: below, pos = 2: left, pos = 3: up, pos = 4: right.

#### Plot (2) ################################################################
# choose some outputs for plot (1)
# response = y3, shock = y1, 
print(length(y3_s1))
print(length(y3_tot))

y_data_31 <- cbind(y3_tot, y3_s1) 
print(y_data_31[1:5, ])

y_min <- min(y_data_31)
y_max <- max(y_data_31)
print(c(y_min, y_max))

# get x_data
num_row <- nrow(y_data_31)
x_data <- matrix(1:num_row, nrow = num_row, ncol = 1)
print(x_data[1:5])
x_min <- min(x_data)
x_max <- max(x_data)
print(c(x_min, x_max))

#### plot for "31" 
plot.new()
matplot(x_data, y_data_31, 
type = "l", lwd = 2, col = 1, 
lty = c("dotted", "solid", "dashed"),
xlim = c(1, (x_max + 40)), 
ylim = c(-5.0, 12.0), 
xlab = "time (quarter) (1 = 1960Q1)", 
ylab = "value (percent)",
main = "response = y3, shock = y1, use of VMA method")

## additional information to the graph
abline(h = 0, col = "gray")
text(x_max, y_data_31[x_data == x_max, 1], labels = "HD_3_tot", pos = 4)
text(x_max, y_data_31[x_data == x_max, 2], labels = "HD_31", pos = 4)
# pos = 1: below, pos = 2: left, pos = 3: up, pos = 4: right.

#### Comparison part ######################################################
# This part is made only for the comparison purpose.
# Delete this part if necessary.
# We use hd() function of Package "svars" for the comparison.

#### We choose: series = "y3" for hd() function
hd_y3 <- hd(x = out_id, series = 3, transition = 0)
plot(hd_y3)

# get better plots
names(hd_y3)
names(hd_y3$hidec)

#### The updated hd() function provides the following 6 output objects. 
V1 <- hd_y3$hidec[, 1]
print(V1[1:5]) 
# This is a time index, as expected

# The following is "demeaned" value
hd_y3_dm <- as.vector(hd_y3$hidec[, 2])

# The following is the "total" of all effects. 
hd_y3_tot <- as.vector(hd_y3$hidec[, 3])

# The following is the effect of "s1" on "y3"
hd_y3_s1 <- as.vector(hd_y3$hidec[, 4])

# The following is the effect of "s2" on "y3"
hd_y3_s2 <- as.vector(hd_y3$hidec[, 5])

# The following is the effect of "s3" on "y3"
hd_y3_s3 <- as.vector(hd_y3$hidec[, 6])

print(hd_y3_dm[1:5])
print(hd_y3_tot[1:5])
print(hd_y3_s1[1:5])
print(hd_y3_s2[1:5])
print(hd_y3_s3[1:5])

#### prepare for plot: "31_hd"
# response = y3, shock = y1, 
# y_data for "31_hd"
y_data_31_hd <- cbind(hd_y3_tot, hd_y3_s1) 
print(y_data_31_hd[1:10, ])

y_min <- min(y_data_31_hd)
y_max <- max(y_data_31_hd)
print(c(y_min, y_max))

# get x_data
num_row <- nrow(y_data_31_hd)
x_data <- matrix(1:num_row, nrow = num_row, ncol = 1)
print(x_data[1:10])
x_min <- min(x_data)
x_max <- max(x_data)
print(c(x_min, x_max))

#### plot for "31_hd" 
plot.new()
matplot(x_data, y_data_31_hd, 
type = "l", lwd = 2, col = 1, 
lty = c("dotted", "solid", "dashed"),
xlim = c(1, (x_max + 40)), 
ylim = c(-5.0, 12.0), 
xlab = "time (quarter) (1 = 1960Q1)", 
ylab = "value (percent)",
main = "response = y3, shock = y1, by hd() function")

#### additional information to the graph
abline(h = 0, col = "gray")
text(x_max, y_data_31_hd[x_data == x_max, 1], labels = "HD_3_tot", pos = 4)
text(x_max, y_data_31_hd[x_data == x_max, 2], labels = "HD_31", pos = 4)
# pos = 1: below, pos = 2: left, pos = 3: up, pos = 4: right.

#### check some values with regard to "hd_y3"
test_hd_y3_tot <- hd_y3_s1 + hd_y3_s2 + hd_y3_s3
print(hd_y3_tot[1:5])
print(test_hd_y3_tot[1:5])

#### Print some values for comparison #############################################
# The comparison is suitable only if we use "Cmat = C_id" and so on.

# for "31"
print(y3_s1[1:5])
print(hd_y3_s1[1:5])

# for "32"
print(y3_s2[1:5])
print(hd_y3_s2[1:5])

# for "33"
print(y3_s3[1:5])
print(hd_y3_s3[1:5])

# for "tot"
print(y3_tot[1:5])
print(hd_y3_tot[1:5])

# for "Cmat"
print(Cmat)
print(out_id$B)

#### end ###############################################################################
