#### R script ########################################
# Last modfied: 2019-09-09
# File name: hd_id_svars.R
# Keywords: historical decomposition, 
#   using hd() function in "svars",
#   using id.dc() function in "svars",
#   using VAR() function in "vars",     
#
# Written by: Hiroshi Murao
#
# Purpose: perform the historical decomposition analysis, 
#     using hd() in Package "svars". 
#
# Name of data set: "Canada" in Package "vars".
#
# Sample periods:  1980:1 2004:4
#
# List of variables
#
# prod : labor productivity = log of GDP - log of employment
# e : log of employment
# U : unemplyment rate
# rw : real wage = log of real wage index 
#
#### first things to do 
setwd("C:/book_VAR/stat_works/Pfaff_Canada/R_prg")
library(vars)
library(svars)

#### data preparations
data(Canada)
dim(Canada)
print("The data set of Canada")
print(Canada[1:5, ])

ys_level <- Canada[, c("prod", "e", "U", "rw")]

Canada_diff <- Canada_diff[, c("prod", "e", "U", "rw")]
print(Canada_diff[1:5, ])

ys_diff <- na.omit(Canada_diff)
print(ys_diff[1:5, ])

#### estimate a VAR model, using VAR() in "vars"
# out_VAR <- VAR(y = ys_level, type = "both", p = 3, ic = "AIC")
out_VAR <- VAR(y = ys_diff, type = "const", p = 2, ic = "AIC")

summary(out_VAR)

#### convert out_VAR to an object for hd()
out_id <- id.dc(x = out_VAR)
summary(out_id)

#### histrical decomposition, using hd() in "svars"
# series = response variable : "U" = y3 
hd_y3 <- hd(x = out_id, series = 3, transition = 0)

names(hd_y3)
print(hd_y3)

#### plot the output of hd()
plot.new()
plot(hd_y3)

#### save the graph as a file
# dev.copy(jpeg, file = "./outputs/hd_y3.jpg") 
# dev.copy(postscript, file = "./outputs/hd_y3.eps") 
# dev.off()

#### prepare for better plot
# series: y3 = "U"
names(hd_y3)
dim(hd_y3$hidec)
# 81 x 6

print(hd_y3$hidec[1:5, ])

## The updated hd() provides (K + 2) vectors of output data,
# excluding the time index. 
# The output of hd() might not have the time index. 

# The following is "demeaned" value of y3
# y3_dm = y3 - mean(y3). 
hd_y3_dm <- as.vector(hd_y3$hidec[, 1])

# The following is the "total" of all effects.
hd_y3_tot <- as.vector(hd_y3$hidec[, 2])
 
# The following is the effect of "s1" on "y3"
hd_y3_s1 <- as.vector(hd_y3$hidec[, 3])

# The following is the effect of "s2" on "y3"
hd_y3_s2 <- as.vector(hd_y3$hidec[, 4])

# The following is the effect of "s3" on "y3"
hd_y3_s3 <- as.vector(hd_y3$hidec[, 5])

# The following is the effect of "s4" on "y3"
hd_y3_s4 <- as.vector(hd_y3$hidec[, 6])

print(hd_y3_dm[1:5])
print(hd_y3_tot[1:5])

#### plot (1) ##################################
# (1) "dm" vs. "tot" 
print(length(hd_y3_dm))
print(length(hd_y3_tot))

y_data_tot <- cbind(hd_y3_dm, hd_y3_tot) 
print(y_data_tot[1:10, ])

y_min <- min(y_data_tot)
y_max <- max(y_data_tot)
print(c(y_min, y_max))

# get x_data
num_row <- nrow(y_data_tot)
x_data <- matrix(1:num_row, nrow = num_row, ncol = 1)
print(x_data[1:10])
x_min <- min(x_data)
x_max <- max(x_data)
print(c(x_min, x_max))

#### plot for y_data_tot 
plot.new()
matplot(x_data, y_data_tot, 
type = "l", lwd = 2, col = 1, 
lty = c("dotted", "solid", "dashed"),
xlim = c(1, (x_max + 20)), 
# ylim = c(-5.0, 5.0), 
ylim = c(-2.0, 2.0), 
xlab = "time (quarter) (1 = 1980Q4)", 
ylab = "value (percent)",
main = "dm vs. tot for y3, use of hd() function")

#### additional information to the graph
abline(h = 0, col = "gray")
text(x_max, y_data_tot[x_data == x_max, 1], labels = "HD_3_rm", pos = 4)
text(x_max, y_data_tot[x_data == x_max, 2], labels = "HD_3_tot", pos = 4)
# pos = 1: below, pos = 2: left, pos = 3: up, pos = 4: right.

#### experiment (1)
# estimate a level-VAR model, using VAR() in "vars"
out_VAR_level <- VAR(y = ys_level, type = "both", p = 3, ic = "AIC")
# out_VAR <- VAR(y = ys_diff, type = "const", p = 2, ic = "AIC")

summary(out_VAR_level)

#### convert out_VAR to an object for hd()
out_id_level <- id.dc(x = out_VAR_level)
summary(out_id_level)

# histrical decomposition, using hd() in "svars"
# series = response variable : "U" = y3 
hd_y3_level <- hd(x = out_id_level, series = 3, transition = 0)

# plot the output of hd()
plot.new()
plot(hd_y3_level)

plot.new()
plot(hd_y3)

#### end #############################################
