#### R script ########################################
# Last modfied: 2019-10-01
# File name: read_KPSW_data.R
# Keywords: read the KPSW data,
#   KPSW = King, Plosser, Stock, and Watson (1991).
#  
# Written by: Hiroshi Murao
#
# Purpose: Read the KPSW data used in the following journal article. 
#
# Source of data set 
#   http://www.princeton.edu/~mwatson/publi.html
#   The data set is known as "KPSW data", 
#   which is used for the following journal article.
#    
# Related journal article
#   King, Plosser, Stock, and Watson (1991), "Stochastic Trends and Economic Fluctuations" 
#   American Economic Review, Vol. 81, No. 4, (September 1991), pp. 819-40.
#
# Sample periods:  1949Q1 - 1988Q4 for regression (page 824).
#                  1947Q1 - 1948Q4 as initial values. 
#                  note the use of eight lags (page 829).
#                  1947Q1 - 1988Q4 as the whole data set. 
#
# List of variables
#
#   Y : per capita "private" gross national product 
#       less real total government purchases of goods and services.
#   C : per capita real consumption expenditures.
#   I : per capita gross private domestic fixed investment.
#   MP : real money supply, logM - logP
#   R : interest rate, the three-month US Treasury bill rate.
#   DP : price inflation rate, 400*log[P(t) / P(t-1)].
#
#### first things to do 
setwd("C:/book_VAR/data_works/data_KPSW/R_prg")
library(vars)
library(urca)
library(tseries)

#### read data 
# C: consumption, I: investment, Y: output, 
# M: real money supply, R: interest rate, DP: inflation rate
data_CIYMRDP <- read.table("./kpsw_zip/DATA/CIYMRDP.PRN", header = F, sep = "")
print(data_CIYMRDP[1:10, ])

# consumption, investment, output
data_CIY <- read.table("./kpsw_zip/DATA/CIY.PRN", header = F, sep = "")
print(data_CIY[1:10, ])

# consumption
data_C <- read.table("./kpsw_zip/DATA/C.PRN", header = F, sep = "")
print(data_C[1:10, ])

# investment
data_IN <- read.table("./kpsw_zip/DATA/IN.PRN", header = F, sep = "")
print(data_IN[1:10, ])

# output
data_Y <- read.table("./kpsw_zip/DATA/Y.PRN", header = F, sep = "")
print(data_Y[1:10, ])

# DP = inflation rate = 400*[logP(t) - logP(t-1)] 
data_DP <- read.table("./kpsw_zip/DATA/DP.PRN", header = F, sep = "")
print(data_DP[1:10, ])

# consumption, output, interest rate, ?,
data_CYR2DP <- read.table("./kpsw_zip/DATA/CYR2DP.PRN", header = F, sep = "")
print(data_CYR2DP[1:10, ])

# interest rate
data_R <- read.table("./kpsw_zip/DATA/R.PRN", header = F, sep = "")
print(data_R[1:10, ])

# real money supply = logM - logP
data_MP <- read.table("./kpsw_zip/DATA/MP.PRN", header = F, sep = "")
print(data_MP[1:10, ])

#### put variable names
myNames <- c("logC", "logI", "logY", "logMP", "interest", "inflation")
names(data_CIYMRDP) <- myNames

#### get the data for use
in_data <- data_CIYMRDP
print(in_data[1:10, ])

#### get the data of (logY, logC, logI, logMP)
# See FIGURE 1 in King at al. (1991)
data_fig1A <- data.frame(y1 = in_data$logY, y2 = in_data$logC, y3 = in_data$logI, y4 = in_data$logMP)

# get x_data
num_obs <- nrow(data_fig1A)
time <- matrix(1:num_obs, nrow = num_obs, ncol = 1)
print(time[1:10])

# make time series, using ts(data, start, frequency, ...)
date <- ts(time, start = c(1947, 1), frequency = 4)
# make sure it starts from 1947Q1 and ends at 1988Q4 
print(date)

# prepare for plot
x_data <- time
x_max <- max(x_data)
x_min <- min(x_data)
print(c(x_min, x_max))

y_max <- max(data_fig1A[, 1])
y_min <- min(data_fig1A[, 4])
print(c(y_min, y_max))

#### plot FIGURE 1(A), using matplot().
plot.new()
matplot(x_data, data_fig1A, type = "l", lwd = 2, col = 1, 
lty = c("solid", "dotted", "dashed", "dashed"), 
xlim = c(0, (num_obs + 30)), 
ylim = c(-7.0, -4.0), 
xlab = "time (quarter)", 
ylab = "logarithms (non-scaled)", 
main = "Time series: (logY, logC, logI, logMP)")

# add curve labels and the like
abline(h = 0, col = "gray")
text(x_max, data_fig1A[(x_data == x_max), 1], labels = "logY", pos = 4)
text(x_max, data_fig1A[(x_data == x_max), 2], labels = "logC", pos = 4)
text(x_max, data_fig1A[(x_data == x_max), 3], labels = "logI", pos = 4)
text(x_max, data_fig1A[(x_data == x_max), 4], labels = "logMP", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

# save the graph as a file
# dev.copy(pdf, file = "outputs/fig_1A_KPSW.pdf")
# dev.off()

#### compute (logC-logY) and (logI-logY)
# See FIGURE 1(B) in King at al. (1991)
y2_y1 <- data_fig1A$y2 - data_fig1A$y1
y3_y1 <- data_fig1A$y3 - data_fig1A$y1

data_fig1B <- cbind(y2_y1, y3_y1)

# prepare for plot
y_max <- max(data_fig1B[, 1])
y_min <- min(data_fig1B[, 2])
print(c(y_min, y_max))

#### plot FIGURE 1(B), using matplot()
plot.new()
matplot(x_data, data_fig1B, type = "l", lwd = 2, col = 1, 
lty = c("solid", "dashed"), 
xlim = c(0, (num_obs + 30)), 
ylim = c(-2.0, 0.0), 
xlab = "time (quarter)", 
ylab = "logarithms (non-scaled)", 
main = "Time series: (logC - logY) and (logI - logY)")

# add curve labels and the like
abline(h = 0, col = "gray")
text(x_max, data_fig1B[(x_data == x_max), 1], labels = "log(C/Y)", pos = 4)
text(x_max, data_fig1B[(x_data == x_max), 2], labels = "log(I/Y)", pos = 4)
# pos = 1: below,  pos = 2: left,  pos = 3: up,  pos = 4: right.

# save the graph as a file
# dev.copy(pdf, file = "outputs/fig_1B_KPSW.pdf")
# dev.off()

#### plot FIGURE 1(A), using plot()
ts_fig1A <- ts(data_fig1A, start = c(1947, 1), frequency = 4)
plot(ts_fig1A, main = "Figure 1(A)")
print(ts_fig1A)

#### plot FIGURE 1(B), using plot()
ts_fig1B <- ts(data_fig1B, start = c(1947, 1), frequency = 4)
plot(ts_fig1B, main = "Figure 1(B)")
names(ts_fig1B)
print(ts_fig1B)

#### end #############################################
